/*
   Project: MPDCon

   Copyright (C) 2004

   Author: Daniel Luederwald

   Created: 2004-06-03 12:54:16 +0200 by flip

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#ifndef _OWNSTRINGS_H_
#define _OWNSTRINGS_H_

static NSString *PlaylistDragType = @"PlaylistDragType";
static NSString *CollectionDragType = @"CollectionDragType";

static NSString *PlaylistChangedNotification = @"PlaylistChangedNotification";
static NSString *SongChangedNotification = @"SongChangedNotification";
static NSString *ShownCollectionChangedNotification = @"ShownCollectionChangedNotification";
static NSString *DidNotConnectNotification = @"DidNotConnectNotification";
static NSString *DidConnectNotification = @"DidConnectNotification";
static NSString *PreferencesChangedNotification = @"PreferencesChangedNotification";
static NSString *GetTracksNotification = @"GetTracksNotification";
static NSString *TracksSendNotification = @"TracksSendNotification";
static NSString *KeyEventNotification = @"KeyEventNotification";
#endif // _OWNSTRINGS_H_

